@@ -5,7 +5,6 @@
  */
 #include <AudioUnit/AudioUnit.h>
 #include <AudioToolbox/AudioToolbox.h>
-#include <pthread.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -16,7 +15,7 @@ struct auplay_st {
 	struct audiosess_st *sess;
 	struct auplay_prm prm;
 	AudioUnit au;
-	pthread_mutex_t mutex;
+	mtx_t mutex;
 	uint32_t sampsz;
 	auplay_write_h *wh;
 	void *arg;
@@ -27,9 +26,9 @@ static void auplay_destructor(void *arg)
 {
 	struct auplay_st *st = arg;
 
-	pthread_mutex_lock(&st->mutex);
+	mtx_lock(&st->mutex);
 	st->wh = NULL;
-	pthread_mutex_unlock(&st->mutex);
+	mtx_unlock(&st->mutex);
 
 	AudioOutputUnitStop(st->au);
 	AudioUnitUninitialize(st->au);
@@ -37,7 +36,7 @@ static void auplay_destructor(void *arg)
 
 	mem_deref(st->sess);
 
-	pthread_mutex_destroy(&st->mutex);
+	mtx_destroy(&st->mutex);
 }
 
 
@@ -58,10 +57,10 @@ static OSStatus output_callback(void *inRefCon,
 	(void)inBusNumber;
 	(void)inNumberFrames;
 
-	pthread_mutex_lock(&st->mutex);
+	mtx_lock(&st->mutex);
 	wh  = st->wh;
 	arg = st->arg;
-	pthread_mutex_unlock(&st->mutex);
+	mtx_unlock(&st->mutex);
 
 	if (!wh)
 		return 0;
@@ -72,8 +71,9 @@ static OSStatus output_callback(void *inRefCon,
 		struct auframe af;
 		uint64_t ts;
 
-		auframe_init(&af, st->prm.fmt,
-			     ab->mData, ab->mDataByteSize/st->sampsz);
+		auframe_init(&af, st->prm.fmt, ab->mData,
+			     ab->mDataByteSize / st->sampsz, st->prm.srate,
+			     st->prm.ch);
 
 		ts = AUDIO_TIMEBASE * inTimeStamp->mSampleTime / st->prm.srate;
 
@@ -131,9 +131,11 @@ int audiounit_player_alloc(struct auplay_st **stp, const struct auplay *ap,
 		goto out;
 	}
 
-	err = pthread_mutex_init(&st->mutex, NULL);
-	if (err)
+	err = mtx_init(&st->mutex, mtx_plain) != thrd_success;
+	if (err) {
+		err = ENOMEM;
 		goto out;
+	}
 
 	err = audiosess_alloc(&st->sess, interrupt_handler, st);
 	if (err)
