@@ -2,8 +2,12 @@
  * @file avcodec/encode.c  Video codecs using libavcodec -- encoder
  *
  * Copyright (C) 2010 - 2013 Alfred E. Heggestad
+ * Copyright (C) 2021 by:
+ *     Media Magic Technologies <developer@mediamagictechnologies.com>
+ *     and Divus GmbH <developer@divus.eu>
  */
 #include <re.h>
+#include <re_h265.h>
 #include <rem.h>
 #include <baresip.h>
 #include <libavcodec/avcodec.h>
@@ -19,14 +23,8 @@ enum {
 };
 
 
-struct picsz {
-	enum h263_fmt fmt;  /**< Picture size */
-	uint8_t mpi;        /**< Minimum Picture Interval (1-32) */
-};
-
-
 struct videnc_state {
-	AVCodec *codec;
+	const AVCodec *codec;
 	AVCodecContext *ctx;
 	struct mbuf *mb_frag;
 	struct videnc_param encprm;
@@ -34,14 +32,9 @@ struct videnc_state {
 	enum vidfmt fmt;
 	enum AVCodecID codec_id;
 	videnc_packet_h *pkth;
-	void *arg;
+	const struct video *vid;
 
 	union {
-		struct {
-			struct picsz picszv[8];
-			uint32_t picszn;
-		} h263;
-
 		struct {
 			uint32_t packetization_mode;
 			uint32_t profile_idc;
@@ -65,7 +58,6 @@ static void destructor(void *arg)
 }
 
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 static int set_hwframe_ctx(AVCodecContext *ctx, AVBufferRef *device_ctx,
 			   int width, int height)
 {
@@ -105,7 +97,6 @@ static int set_hwframe_ctx(AVCodecContext *ctx, AVBufferRef *device_ctx,
 
 	return err;
 }
-#endif
 
 
 static enum AVPixelFormat vidfmt_to_avpixfmt(enum vidfmt fmt)
@@ -116,51 +107,12 @@ static enum AVPixelFormat vidfmt_to_avpixfmt(enum vidfmt fmt)
 	case VID_FMT_YUV444P: return AV_PIX_FMT_YUV444P;
 	case VID_FMT_NV12:    return AV_PIX_FMT_NV12;
 	case VID_FMT_NV21:    return AV_PIX_FMT_NV21;
+	case VID_FMT_YUV422P: return AV_PIX_FMT_YUV422P;
 	default:              return AV_PIX_FMT_NONE;
 	}
 }
 
 
-static enum h263_fmt h263_fmt(const struct pl *name)
-{
-	if (0 == pl_strcasecmp(name, "sqcif")) return H263_FMT_SQCIF;
-	if (0 == pl_strcasecmp(name, "qcif"))  return H263_FMT_QCIF;
-	if (0 == pl_strcasecmp(name, "cif"))   return H263_FMT_CIF;
-	if (0 == pl_strcasecmp(name, "cif4"))  return H263_FMT_4CIF;
-	if (0 == pl_strcasecmp(name, "cif16")) return H263_FMT_16CIF;
-	return H263_FMT_OTHER;
-}
-
-
-static int decode_sdpparam_h263(struct videnc_state *st, const struct pl *name,
-				const struct pl *val)
-{
-	enum h263_fmt fmt = h263_fmt(name);
-	const int mpi = pl_u32(val);
-
-	if (fmt == H263_FMT_OTHER) {
-		info("h263: unknown param '%r'\n", name);
-		return 0;
-	}
-	if (mpi < 1 || mpi > 32) {
-		info("h263: %r: MPI out of range %d\n", name, mpi);
-		return 0;
-	}
-
-	if (st->u.h263.picszn >= ARRAY_SIZE(st->u.h263.picszv)) {
-		info("h263: picszv overflow: %r\n", name);
-		return 0;
-	}
-
-	st->u.h263.picszv[st->u.h263.picszn].fmt = fmt;
-	st->u.h263.picszv[st->u.h263.picszn].mpi = mpi;
-
-	++st->u.h263.picszn;
-
-	return 0;
-}
-
-
 static int init_encoder(struct videnc_state *st, const char *name)
 {
 	/*
@@ -198,6 +150,7 @@ static int open_encoder(struct videnc_state *st,
 			int pix_fmt)
 {
 	int err = 0;
+	uint32_t keyint = KEYFRAME_INTERVAL;
 
 	if (st->ctx)
 		avcodec_free_context(&st->ctx);
@@ -210,20 +163,23 @@ static int open_encoder(struct videnc_state *st,
 
 	av_opt_set_defaults(st->ctx);
 
-	st->ctx->bit_rate  = prm->bitrate;
+	st->ctx->bit_rate	= prm->bitrate;
+	st->ctx->rc_max_rate	= prm->bitrate;
+	st->ctx->rc_buffer_size = prm->bitrate / 2;
+
 	st->ctx->width     = size->w;
 	st->ctx->height    = size->h;
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	if (avcodec_hw_type == AV_HWDEVICE_TYPE_VAAPI)
 		st->ctx->pix_fmt   = avcodec_hw_pix_fmt;
 	else
-#endif
 		st->ctx->pix_fmt   = pix_fmt;
 
+	conf_get_u32(conf_cur(), "avcodec_keyint", &keyint);
+
 	st->ctx->time_base.num = 1;
 	st->ctx->time_base.den = prm->fps;
-	st->ctx->gop_size = KEYFRAME_INTERVAL * prm->fps;
+	st->ctx->gop_size = keyint * prm->fps;
 
 	if (0 == str_cmp(st->codec->name, "libx264")) {
 
@@ -287,7 +243,6 @@ static int open_encoder(struct videnc_state *st,
 		av_opt_set(st->ctx->priv_data, "tune", "zerolatency", 0);
 	}
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	if (avcodec_hw_type == AV_HWDEVICE_TYPE_VAAPI) {
 
 		/* set hw_frames_ctx for encoder's AVCodecContext */
@@ -301,7 +256,6 @@ static int open_encoder(struct videnc_state *st,
 			goto out;
 		}
 	}
-#endif
 
 	if (avcodec_open2(st->ctx, st->codec, NULL) < 0) {
 		err = ENOENT;
@@ -362,63 +316,15 @@ static void param_handler(const struct pl *name, const struct pl *val,
 {
 	struct videnc_state *st = arg;
 
-	if (st->codec_id == AV_CODEC_ID_H263)
-		(void)decode_sdpparam_h263(st, name, val);
-	else if (st->codec_id == AV_CODEC_ID_H264)
+	if (st->codec_id == AV_CODEC_ID_H264)
 		(void)decode_sdpparam_h264(st, name, val);
 }
 
 
-static int h263_packetize(struct videnc_state *st,
-			  uint64_t rtp_ts, struct mbuf *mb,
-			  videnc_packet_h *pkth, void *arg)
-{
-	struct h263_strm h263_strm;
-	struct h263_hdr h263_hdr;
-	size_t pos;
-	int err;
-
-	/* Decode bit-stream header, used by packetizer */
-	err = h263_strm_decode(&h263_strm, mb);
-	if (err)
-		return err;
-
-	h263_hdr_copy_strm(&h263_hdr, &h263_strm);
-
-	st->mb_frag->pos = st->mb_frag->end = 0;
-	err = h263_hdr_encode(&h263_hdr, st->mb_frag);
-	pos = st->mb_frag->pos;
-
-	/* Assemble frame into smaller packets */
-	while (!err) {
-		size_t sz, left = mbuf_get_left(mb);
-		bool last = (left < st->encprm.pktsize);
-		if (!left)
-			break;
-
-		sz = last ? left : st->encprm.pktsize;
-
-		st->mb_frag->pos = st->mb_frag->end = pos;
-		err = mbuf_write_mem(st->mb_frag, mbuf_buf(mb), sz);
-		if (err)
-			break;
-
-		st->mb_frag->pos = 0;
-
-		err = pkth(last, rtp_ts, NULL, 0, mbuf_buf(st->mb_frag),
-			   mbuf_get_left(st->mb_frag), arg);
-
-		mbuf_advance(mb, sz);
-	}
-
-	return err;
-}
-
-
 int avcodec_encode_update(struct videnc_state **vesp,
-			  const struct vidcodec *vc,
-			  struct videnc_param *prm, const char *fmtp,
-			  videnc_packet_h *pkth, void *arg)
+			  const struct vidcodec *vc, struct videnc_param *prm,
+			  const char *fmtp, videnc_packet_h *pkth,
+			  const struct video *vid)
 {
 	struct videnc_state *st;
 	int err = 0;
@@ -434,8 +340,8 @@ int avcodec_encode_update(struct videnc_state **vesp,
 		return ENOMEM;
 
 	st->encprm = *prm;
-	st->pkth = pkth;
-	st->arg = arg;
+	st->pkth   = pkth;
+	st->vid	   = vid;
 
 	st->codec_id = avcodec_resolve_codecid(vc->name);
 	if (st->codec_id == AV_CODEC_ID_NONE) {
@@ -486,11 +392,7 @@ int avcodec_encode(struct videnc_state *st, bool update,
 	AVFrame *hw_frame = NULL;
 	AVPacket *pkt = NULL;
 	int i, err = 0, ret;
-#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(57, 37, 100)
-	int got_packet = 0;
-#endif
 	uint64_t ts;
-	struct mbuf mb;
 
 	if (!st || !frame)
 		return EINVAL;
@@ -522,7 +424,6 @@ int avcodec_encode(struct videnc_state *st, bool update,
 		goto out;
 	}
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	if (avcodec_hw_type == AV_HWDEVICE_TYPE_VAAPI) {
 		hw_frame = av_frame_alloc();
 		if (!hw_frame) {
@@ -530,7 +431,6 @@ int avcodec_encode(struct videnc_state *st, bool update,
 			goto out;
 		}
 	}
-#endif
 
 	pict->format = vidfmt_to_avpixfmt(frame->fmt);
 	pict->width = frame->size.w;
@@ -544,15 +444,16 @@ int avcodec_encode(struct videnc_state *st, bool update,
 
 	if (update) {
 		debug("avcodec: encoder picture update\n");
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(58, 29, 100)
+		pict->flags |= AV_FRAME_FLAG_KEY;
+#else
 		pict->key_frame = 1;
+#endif
 		pict->pict_type = AV_PICTURE_TYPE_I;
 	}
 
-#if LIBAVUTIL_VERSION_MAJOR >= 55
 	pict->color_range = AVCOL_RANGE_MPEG;
-#endif
 
-#if LIBAVUTIL_VERSION_MAJOR >= 56
 	if (avcodec_hw_type == AV_HWDEVICE_TYPE_VAAPI) {
 
 		if ((err = av_hwframe_get_buffer(st->ctx->hw_frames_ctx,
@@ -576,9 +477,6 @@ int avcodec_encode(struct videnc_state *st, bool update,
 
 		av_frame_copy_props(hw_frame, pict);
 	}
-#endif
-
-#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 37, 100)
 
 	pkt = av_packet_alloc();
 	if (!pkt) {
@@ -597,53 +495,22 @@ int avcodec_encode(struct videnc_state *st, bool update,
 		err = 0;
 		goto out;
 	}
-#else
-
-	pkt = av_malloc(sizeof(*pkt));
-	if (!pkt) {
-		err = ENOMEM;
-		goto out;
-	}
-
-	av_init_packet(pkt);
-	av_new_packet(pkt, 65536);
-
-	ret = avcodec_encode_video2(st->ctx, pkt, pict, &got_packet);
-	if (ret < 0) {
-		err = EBADMSG;
-		goto out;
-	}
-
-	if (!got_packet)
-		return 0;
-#endif
-
-	mb.buf = pkt->data;
-	mb.pos = 0;
-	mb.end = pkt->size;
-	mb.size = pkt->size;
 
 	ts = video_calc_rtp_timestamp_fix(pkt->pts);
 
 	switch (st->codec_id) {
 
-	case AV_CODEC_ID_H263:
-		err = h263_packetize(st, ts, &mb, st->pkth, st->arg);
-		break;
-
 	case AV_CODEC_ID_H264:
-		err = h264_packetize(ts, pkt->data, pkt->size,
-				     st->encprm.pktsize,
-				     st->pkth, st->arg);
+		err = h264_packetize(
+			ts, pkt->data, pkt->size, st->encprm.pktsize,
+			(h264_packet_h *)st->pkth, (void *)st->vid);
 		break;
 
-#ifdef AV_CODEC_ID_H265
 	case AV_CODEC_ID_H265:
-		err = h265_packetize(ts, pkt->data, pkt->size,
-				     st->encprm.pktsize,
-				     st->pkth, st->arg);
+		err = h265_packetize(
+			ts, pkt->data, pkt->size, st->encprm.pktsize,
+			(h265_packet_h *)st->pkth, (void *)st->vid);
 		break;
-#endif
 
 	default:
 		err = EPROTO;
@@ -659,3 +526,36 @@ int avcodec_encode(struct videnc_state *st, bool update,
 
 	return err;
 }
+
+
+int avcodec_packetize(struct videnc_state *st, const struct vidpacket *packet)
+{
+	int err = 0;
+	uint64_t ts;
+
+	if (!st || !packet)
+		return EINVAL;
+
+	ts = video_calc_rtp_timestamp_fix(packet->timestamp);
+
+	switch (st->codec_id) {
+
+	case AV_CODEC_ID_H264:
+		err = h264_packetize(
+			ts, packet->buf, packet->size, st->encprm.pktsize,
+			(h264_packet_h *)st->pkth, (void *)st->vid);
+		break;
+
+	case AV_CODEC_ID_H265:
+		err = h265_packetize(
+			ts, packet->buf, packet->size, st->encprm.pktsize,
+			(h265_packet_h *)st->pkth, (void *)st->vid);
+		break;
+
+	default:
+		err = EPROTO;
+		break;
+	}
+
+	return err;
+}
