@@ -5,7 +5,6 @@
  */
 #include <AudioToolbox/AudioQueue.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -18,7 +17,7 @@
 struct ausrc_st {
 	AudioQueueRef queue;
 	AudioQueueBufferRef buf[BUFC];
-	pthread_mutex_t mutex;
+	mtx_t mutex;
 	struct ausrc_prm prm;
 	uint32_t sampsz;
 	int fmt;
@@ -32,22 +31,22 @@ static void ausrc_destructor(void *arg)
 	struct ausrc_st *st = arg;
 	uint32_t i;
 
-	pthread_mutex_lock(&st->mutex);
+	mtx_lock(&st->mutex);
 	st->rh = NULL;
-	pthread_mutex_unlock(&st->mutex);
+	mtx_unlock(&st->mutex);
 
 	if (st->queue) {
 		AudioQueuePause(st->queue);
 		AudioQueueStop(st->queue, true);
 
-		for (i=0; i<ARRAY_SIZE(st->buf); i++)
+		for (i=0; i<RE_ARRAY_SIZE(st->buf); i++)
 			if (st->buf[i])
 				AudioQueueFreeBuffer(st->queue, st->buf[i]);
 
 		AudioQueueDispose(st->queue, true);
 	}
 
-	pthread_mutex_destroy(&st->mutex);
+	mtx_destroy(&st->mutex);
 }
 
 
@@ -65,16 +64,17 @@ static void record_handler(void *userData, AudioQueueRef inQ,
 	(void)inNumPackets;
 	(void)inPacketDesc;
 
-	pthread_mutex_lock(&st->mutex);
+	mtx_lock(&st->mutex);
 	rh  = st->rh;
 	arg = st->arg;
-	pthread_mutex_unlock(&st->mutex);
+	mtx_unlock(&st->mutex);
 
 	if (!rh)
 		return;
 
-	auframe_init(&af, st->fmt, inQB->mAudioData,
-		     inQB->mAudioDataByteSize/st->sampsz);
+	auframe_init(&af, st->prm.fmt, inQB->mAudioData,
+		     inQB->mAudioDataByteSize / st->sampsz, st->prm.srate,
+		     st->prm.ch);
 
 	af.timestamp = AUDIO_TIMEBASE*inStartTime->mSampleTime / st->prm.srate;
 
@@ -85,7 +85,6 @@ static void record_handler(void *userData, AudioQueueRef inQ,
 
 
 int coreaudio_recorder_alloc(struct ausrc_st **stp, const struct ausrc *as,
-			     struct media_ctx **ctx,
 			     struct ausrc_prm *prm, const char *device,
 			     ausrc_read_h *rh, ausrc_error_h *errh, void *arg)
 {
@@ -95,7 +94,6 @@ int coreaudio_recorder_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	OSStatus status;
 	int err;
 
-	(void)ctx;
 	(void)errh;
 
 	if (!stp || !as || !prm)
@@ -119,9 +117,11 @@ int coreaudio_recorder_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	st->fmt = prm->fmt;
 	st->prm = *prm;
 
-	err = pthread_mutex_init(&st->mutex, NULL);
-	if (err)
+	err = mtx_init(&st->mutex, mtx_plain) != thrd_success;
+	if (err) {
+		err = ENOMEM;
 		goto out;
+	}
 
 	fmt.mSampleRate       = (Float64)prm->srate;
 	fmt.mFormatID         = kAudioFormatLinearPCM;
@@ -175,7 +175,7 @@ int coreaudio_recorder_alloc(struct ausrc_st **stp, const struct ausrc *as,
 		}
 	}
 
-	for (i=0; i<ARRAY_SIZE(st->buf); i++)  {
+	for (i=0; i<RE_ARRAY_SIZE(st->buf); i++)  {
 
 		status = AudioQueueAllocateBuffer(st->queue, bytc,
 						  &st->buf[i]);
