@@ -7,62 +7,19 @@
 #include <re.h>
 #include <baresip.h>
 #include <stdlib.h>
-#include <pthread.h>
 #include <gtk/gtk.h>
 #include "gtk_mod.h"
+#include <ctype.h>
 
 
 struct dial_dialog {
 	struct gtk_mod *mod;
 	GtkWidget *dialog;
 	GtkComboBox *uri_combobox;
+	struct call *attended_call;
 };
 
 
-static int clean_number(char* str)
-{
-	int i = 0, k = 0;
-
-	/* only clean numeric numbers
-	 * In other cases trust the user input
-	 */
-	int err = re_regex(str, str_len(str), "[A-Za-z]");
-	if (err == 0)
-		return -1;
-
-	/* remove (0) which is in some mal-formated numbers
-	 * but only if trailed by another character
-	 */
-	if (str[0] == '+' || (str[0] == '0' && str[1] == '0'))
-		while (str[i]) {
-			if (str[i] == '('
-			 && str[i+1] == '0'
-			 && str[i+2] == ')'
-			 && (str[i+3] == ' '
-				 || (str[i+3] >= '0' && str[i+3] <= '9')
-			    )
-			) {
-				str[i+1] = ' ';
-				break;
-			}
-			++i;
-		}
-	i = 0;
-	while (str[i]) {
-		if (str[i] == ' '
-		 || str[i] == '.'
-		 || str[i] == '-'
-		 || str[i] == '/'
-		 || str[i] == '('
-		 || str[i] == ')')
-			++i;
-		else
-			str[k++] = str[i++];
-	}
-	str[k] = '\0';
-	return k;
-}
-
 static void dial_dialog_on_response(GtkDialog *dialog, gint response_id,
 				    gpointer arg)
 {
@@ -77,7 +34,13 @@ static void dial_dialog_on_response(GtkDialog *dialog, gint response_id,
 				uri_combo_box_set_text(dd->uri_combobox,
 					uri, length);
 		}
-		gtk_mod_connect(dd->mod, uri);
+		if (!dd->attended_call) {
+			gtk_mod_connect(dd->mod, uri);
+		}
+		else {
+			gtk_mod_connect_attended(dd->mod, uri,
+							dd->attended_call);
+		}
 	}
 
 	gtk_widget_hide(GTK_WIDGET(dialog));
@@ -92,7 +55,8 @@ static void destructor(void *arg)
 }
 
 
-struct dial_dialog *dial_dialog_alloc(struct gtk_mod *mod)
+struct dial_dialog *dial_dialog_alloc(struct gtk_mod *mod,
+				struct call *attended_call)
 {
 	struct dial_dialog *dd;
 	GtkWidget *dial;
@@ -138,6 +102,7 @@ struct dial_dialog *dial_dialog_alloc(struct gtk_mod *mod)
 	dd->dialog = dial;
 	dd->uri_combobox = GTK_COMBO_BOX(uri_combobox);
 	dd->mod = mod;
+	dd->attended_call = attended_call;
 
 	return dd;
 }
