@@ -75,9 +75,6 @@ static int session_alloc(struct menc_sess **sessp, struct sdp_session *sdp,
 {
 	struct menc_sess *st;
 	(void)offerer;
-	(void)eventh;
-	(void)errorh;
-	(void)arg;
 	int err = 0;
 
 	if (!sessp || !sdp)
@@ -91,10 +88,15 @@ static int session_alloc(struct menc_sess **sessp, struct sdp_session *sdp,
 	if (!st->session)
 		err = ENOMEM;
 
-	if (err)
+	if (err) {
 		mem_deref(st);
-	else
+	}
+	else {
+		st->session->eventh = eventh;
+		st->session->errorh = errorh;
+		st->session->arg = arg;
 		*sessp = st;
+	}
 
 	return err;
 }
@@ -112,6 +114,8 @@ static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 	int err = 0;
 	StreamMediaType med_type;
 	const char *med_name;
+	(void)raddr_rtp;
+	(void)raddr_rtcp;
 
 	if (!stp || !sess || !sess->session)
 		return EINVAL;
@@ -205,7 +209,7 @@ static int module_init(void)
 
 	menc_register(baresip_mencl(), &menc_zrtp);
 
-	return cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
+	return cmd_register(baresip_commands(), cmdv, RE_ARRAY_SIZE(cmdv));
 }
 
 
