@@ -21,6 +21,7 @@ struct auplay_st {
 	int16_t *sampv[N_PLAY_QUEUE_BUFFERS];
 	size_t   sampc;
 	uint8_t  bufferId;
+	struct auplay_prm prm;
 
 	SLObjectItf outputMixObject;
 	SLObjectItf bqPlayerObject;
@@ -51,7 +52,8 @@ static void bqPlayerCallback(SLAndroidSimpleBufferQueueItf bq, void *context)
 	struct auplay_st *st = context;
 	struct auframe af;
 
-	auframe_init(&af, AUFMT_S16LE, st->sampv[st->bufferId], st->sampc);
+	auframe_init(&af, AUFMT_S16LE, st->sampv[st->bufferId], st->sampc,
+		     st->prm.srate, st->prm.ch);
 
 	st->wh(&af, st->arg);
 
@@ -109,7 +111,7 @@ static int createPlayer(struct auplay_st *st, struct auplay_prm *prm)
 	r = (*engineEngine)->CreateAudioPlayer(engineEngine,
 					       &st->bqPlayerObject,
 					       &audioSrc, &audioSnk,
-					       ARRAY_SIZE(ids), ids, req);
+					       RE_ARRAY_SIZE(ids), ids, req);
 	if (SL_RESULT_SUCCESS != r) {
 		warning("opensles: CreateAudioPlayer error: r = %d\n", r);
 		return ENODEV;
@@ -172,6 +174,7 @@ int opensles_player_alloc(struct auplay_st **stp, const struct auplay *ap,
 
 	st->wh  = wh;
 	st->arg = arg;
+	st->prm = *prm;
 
 	st->sampc = prm->srate * prm->ch * PTIME / 1000;
 
