@@ -4,9 +4,6 @@
  * Copyright (C) 2010 Alfred E. Heggestad
  * \ref GenericAudioStream
  */
-#define _DEFAULT_SOURCE 1
-#define _BSD_SOURCE 1
-#define _POSIX_C_SOURCE 199309L
 #include <string.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -131,7 +128,7 @@ struct audio {
 	bool started;                 /**< Stream is started flag          */
 	bool level_enabled;           /**< Audio level RTP ext. enabled    */
 	bool hold;                    /**< Local hold flag                 */
-	bool conference;              /**< Local conference flag           */
+	RE_ATOMIC bool conference;    /**< Local conference flag           */
 	uint8_t extmap_aulevel;       /**< ID Range 1-14 inclusive         */
 	audio_event_h *eventh;        /**< Event handler                   */
 	audio_level_h *levelh;        /**< Audio level handler             */
@@ -180,6 +177,7 @@ static void stop_tx(struct autx *tx, struct audio *a)
 	if (!tx || !a)
 		return;
 
+	stream_enable_tx(a->strm, false);
 	if (a->cfg.txmode == AUDIO_MODE_THREAD &&
 	    re_atomic_rlx(&tx->thr.run)) {
 		re_atomic_rlx_set(&tx->thr.run, false);
@@ -194,25 +192,15 @@ static void stop_tx(struct autx *tx, struct audio *a)
 }
 
 
-static void stop_aur(struct audio_recv *aur)
-{
-	if (!aur)
-		return;
-
-	/* audio player must be stopped first */
-	aurecv_stop(aur);
-}
-
-
 static void audio_destructor(void *arg)
 {
 	struct audio *a = arg;
 
 	debug("audio: destroyed (started=%d)\n", a->started);
 
-	stream_enable(a->strm, false);
 	stop_tx(&a->tx, a);
-	stop_aur(a->aur);
+	stream_enable_rx(a->strm, false);
+	aurecv_stop(a->aur);
 
 	mem_deref(a->tx.enc);
 	mem_deref(a->tx.aubuf);
@@ -422,7 +410,6 @@ static void poll_aubuf_tx(struct audio *a)
 {
 	struct autx *tx = &a->tx;
 	struct auframe af;
-	int16_t *sampv = tx->sampv;
 	size_t sampc;
 	size_t sz;
 	struct le *le;
@@ -439,9 +426,10 @@ static void poll_aubuf_tx(struct audio *a)
 	ch = tx->ausrc_prm.ch;
 
 	/* timed read from audio-buffer */
-	auframe_init(&af, tx->src_fmt, sampv, sampc, srate, ch);
+	auframe_init(&af, tx->src_fmt, tx->sampv, sampc, srate, ch);
 	aubuf_read_auframe(tx->aubuf, &af);
 
+	mtx_lock(tx->mtx);
 	/* Process exactly one audio-frame in list order */
 	for (le = tx->filtl.head; le; le = le->next) {
 		struct aufilt_enc_st *st = le->data;
@@ -449,6 +437,7 @@ static void poll_aubuf_tx(struct audio *a)
 		if (st->af && st->af->ench)
 			err |= st->af->ench(st, &af);
 	}
+	mtx_unlock(tx->mtx);
 	if (err) {
 		warning("audio: aufilter encode: %m\n", err);
 	}
@@ -520,6 +509,16 @@ bool audio_txtelev_empty(const struct audio *au)
 }
 
 
+void audio_call_telev_handler(const struct audio *au, int key, bool end)
+{
+	if (!au)
+		return;
+
+	if (au->eventh)
+		au->eventh(key, end, au->arg);
+}
+
+
 /*
  * Read samples from Audio Source
  *
@@ -560,7 +559,10 @@ static void ausrc_read_handler(struct auframe *af, void *arg)
 	(void)aubuf_write_auframe(tx->aubuf, af);
 
 	mtx_lock(tx->mtx);
-	tx->aubuf_started = true;
+	if (!tx->aubuf_started &&
+	    (aubuf_cur_size(tx->aubuf) >= tx->psize))
+		tx->aubuf_started = true;
+
 	mtx_unlock(tx->mtx);
 
 	if (a->cfg.txmode != AUDIO_MODE_POLL)
@@ -773,18 +775,6 @@ int audio_alloc(struct audio **ap, struct list *streaml,
 	if (err)
 		goto out;
 
-	if (cfg->audio.level && offerer) {
-
-		a->extmap_aulevel = stream_generate_extmap_id(a->strm);
-		aurecv_set_extmap(a->aur, a->extmap_aulevel);
-
-		err = sdp_media_set_lattr(stream_sdpmedia(a->strm), true,
-					  "extmap",
-					  "%u %s",
-					  a->extmap_aulevel, uri_aulevel);
-		if (err)
-			goto out;
-	}
 
 	tx->mb = mbuf_alloc(STREAM_PRESZ + 4096);
 	tx->sampv = mem_zalloc(AUDIO_SAMPSZ * aufmt_sample_size(tx->enc_fmt),
@@ -828,6 +818,8 @@ int audio_alloc(struct audio **ap, struct list *streaml,
 	if (acc && acc->auplay_mod) {
 		err  = aurecv_set_module(a->aur, acc->auplay_mod);
 		err |= aurecv_set_device(a->aur, acc->auplay_dev);
+		if (err)
+			goto out;
 
 		info("audio: using account specific player: (%s,%s)\n",
 		     acc->auplay_mod, acc->auplay_dev);
@@ -858,6 +850,21 @@ int audio_alloc(struct audio **ap, struct list *streaml,
 }
 
 
+int audio_enable_level(struct audio *au)
+{
+	if (!au)
+		return EINVAL;
+
+	au->extmap_aulevel = stream_generate_extmap_id(au->strm);
+	aurecv_set_extmap(au->aur, au->extmap_aulevel);
+
+	return sdp_media_set_lattr(stream_sdpmedia(au->strm), false,
+				   "extmap",
+				   "%u %s",
+				   au->extmap_aulevel, uri_aulevel);
+}
+
+
 static int tx_thread(void *arg)
 {
 	struct audio *a = arg;
@@ -994,6 +1001,9 @@ static int aufilt_setup(struct audio *a, struct list *aufiltl)
 		struct aufilt_dec_st *decst = NULL;
 		void *ctx = NULL;
 
+		if (!af->enabled)
+			continue;
+
 		if (af->encupdh && update_enc) {
 			err = af->encupdh(&encst, &ctx, af, &encprm, a);
 			if (err) {
@@ -1002,7 +1012,9 @@ static int aufilt_setup(struct audio *a, struct list *aufiltl)
 			}
 			else {
 				encst->af = af;
+				mtx_lock(tx->mtx);
 				list_append(&tx->filtl, &encst->le, encst);
+				mtx_unlock(tx->mtx);
 			}
 		}
 
@@ -1052,20 +1064,20 @@ static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 	/* Start Audio Source */
 	if (!tx->ausrc && ausrc_find(ausrcl, NULL) && !a->hold) {
 
-		struct ausrc_prm prm;
 		size_t sz;
 		size_t psize_alloc;
-
-		prm.srate      = srate_dsp;
-		prm.ch         = channels_dsp;
-		prm.ptime      = tx->ptime;
-		prm.fmt        = tx->src_fmt;
+		struct ausrc_prm prm = {
+			.srate      = srate_dsp,
+			.ch         = channels_dsp,
+			.ptime      = tx->ptime,
+			.fmt        = tx->src_fmt
+		};
 
 		tx->ausrc_prm = prm;
 
 		sz = aufmt_sample_size(tx->src_fmt);
 
-		psize_alloc = sz * calc_nsamp(prm.srate, prm.ch, prm.ptime);
+		psize_alloc = sz * au_calc_nsamp(prm.srate, prm.ch, prm.ptime);
 		tx->psize = psize_alloc;
 		tx->aubuf_maxsz = tx->psize * 30;
 
@@ -1090,7 +1102,7 @@ static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 		/* recalculate and resize aubuf if ausrc_alloc changes prm */
 		tx->src_fmt = prm.fmt;
 		sz = aufmt_sample_size(tx->src_fmt);
-		tx->psize = sz * calc_nsamp(prm.srate, prm.ch, prm.ptime);
+		tx->psize = sz * au_calc_nsamp(prm.srate, prm.ch, prm.ptime);
 		if (psize_alloc != tx->psize) {
 			tx->ausrc_prm = prm;
 			tx->aubuf_maxsz = tx->psize * 30;
@@ -1134,6 +1146,8 @@ static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 		     aufmt_name(tx->src_fmt));
 	}
 
+	stream_enable_tx(a->strm, true);
+
 	return 0;
 }
 
@@ -1151,25 +1165,47 @@ static void audio_flush_filters(struct audio *a)
 
 
 /**
- * Start the audio playback and recording
+ * Update audio object and start/stop according to media direction
  *
  * @param a Audio object
  *
  * @return 0 if success, otherwise errorcode
  */
-int audio_start(struct audio *a)
+int audio_update(struct audio *a)
 {
 	struct list *aufiltl = baresip_aufiltl();
-	const struct sdp_media *m;
-	enum sdp_dir dir;
+	struct sdp_media *m;
+	enum sdp_dir dir = SDP_INACTIVE;
+	const struct sdp_format *sc = NULL;
 	int err = 0;
 
 	if (!a)
 		return EINVAL;
 
-	debug("audio: start\n");
+	debug("audio: update\n");
 	m = stream_sdpmedia(audio_strm(a));
-	dir = sdp_media_dir(m);
+
+	if (!sdp_media_disabled(m)) {
+		dir = sdp_media_dir(m);
+		sc = sdp_media_rformat(m, NULL);
+	}
+
+	if (!sc || !sc->data) {
+		info("audio: stream is disabled\n");
+		audio_stop(a);
+		return 0;
+	}
+
+	if (dir & SDP_RECVONLY)
+		err |= audio_decoder_set(a, sc->data, sc->pt, sc->rparams);
+
+	if (dir & SDP_SENDONLY)
+		err |= audio_encoder_set(a, sc->data, sc->pt, sc->params);
+
+	if (err) {
+		warning("audio: start error (%m)\n", err);
+		return err;
+	}
 
 	/* Audio filter */
 	if (!list_isempty(aufiltl)) {
@@ -1179,14 +1215,19 @@ int audio_start(struct audio *a)
 			return err;
 	}
 
-	if (dir & SDP_RECVONLY)
-		err |= aurecv_start_player(a->aur, baresip_auplayl());
+	if (dir & SDP_RECVONLY) {
+		stream_enable_rx(a->strm, true);
+	}
+	else {
+		stream_enable_rx(a->strm, false);
+		aurecv_stop(a->aur);
+	}
 
-	if (dir & SDP_SENDONLY)
+	if (dir & SDP_SENDONLY) {
 		err |= start_source(&a->tx, a, baresip_ausrcl());
-	if (err) {
-		warning("audio: start error (%m)\n", err);
-		return err;
+	}
+	else {
+		stop_tx(&a->tx, a);
 	}
 
 	if (a->tx.ac && aurecv_codec(a->aur)) {
@@ -1256,8 +1297,8 @@ void audio_stop(struct audio *a)
 		return;
 
 	stop_tx(&a->tx, a);
-	stream_enable(a->strm, false);
-	stop_aur(a->aur);
+	stream_enable_rx(a->strm, false);
+	aurecv_stop(a->aur);
 	a->started = false;
 }
 
@@ -1281,6 +1322,8 @@ bool audio_started(const struct audio *a)
 /**
  * Set the audio encoder used
  *
+ * @note The audio source has to be started separately
+ *
  * @param a      Audio object
  * @param ac     Audio codec to use
  * @param pt_tx  Payload type for sending
@@ -1338,10 +1381,6 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 	if (ac->ptime)
 		tx->ptime = ac->ptime;
 
-	if (!tx->ausrc) {
-		err |= audio_start(a);
-	}
-
 	return err;
 }
 
@@ -1349,6 +1388,8 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 /**
  * Set the audio decoder used
  *
+ * @note Starts also the player if not already running
+ *
  * @param a      Audio object
  * @param ac     Audio codec to use
  * @param pt     Payload type for receiving
@@ -1384,7 +1425,7 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 
 	stream_set_srate(a->strm, 0, ac->crate);
 	if (reset || !aurecv_player_started(a->aur))
-		err |= audio_start(a);
+		err |= aurecv_start_player(a->aur, baresip_auplayl());
 
 	return err;
 }
@@ -1539,7 +1580,7 @@ void audio_sdp_attr_decode(struct audio *a)
 
 				sz = aufmt_sample_size(tx->src_fmt);
 
-				tx->psize = sz * calc_nsamp(tx->ac->srate,
+				tx->psize = sz * au_calc_nsamp(tx->ac->srate,
 							    tx->ac->ch,
 							    ptime_tx);
 			}
@@ -1621,7 +1662,7 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 	tx = &a->tx;
 	sztx = aufmt_sample_size(tx->src_fmt);
 
-	err  = re_hprintf(pf, "\n--- Audio stream ---\n");
+	err  = re_hprintf(pf, "%s", "\n--- Audio stream ---\n");
 
 	err |= re_hprintf(pf, " tx:   encode: %H ptime=%ums %s\n",
 			  aucodec_print, tx->ac,
@@ -1855,7 +1896,7 @@ int audio_set_conference(struct audio *au, bool conference)
 	if (!au)
 		return EINVAL;
 
-	au->conference = conference;
+	re_atomic_rlx_set(&au->conference, conference);
 
 	return 0;
 }
@@ -1870,7 +1911,7 @@ int audio_set_conference(struct audio *au, bool conference)
  */
 bool audio_is_conference(const struct audio *au)
 {
-	return au ? au->conference : false;
+	return au ? re_atomic_rlx(&au->conference) : false;
 }
 
 
