@@ -18,8 +18,8 @@
 #define MAGIC_CHECK(s) \
 	if (MAGIC != s->magic) {					\
 		warning("%s: wrong magic struct=%p (magic=0x%08x)\n",	\
-			__REFUNC__, s, s->magic);			\
-		BREAKPOINT;						\
+			__func__, s, s->magic);			\
+		re_assert(false);					\
 	}
 #else
 #define MAGIC_DECL
