@@ -55,22 +55,21 @@ static void test_abort(struct test *t, int err)
 }
 
 
-static void ua_event_handler(struct ua *ua, enum ua_event ev,
-			     struct call *call, const char *prm, void *arg)
+static void event_handler(enum bevent_ev ev, struct bevent *event, void *arg)
 {
 	struct test *t = arg;
 	size_t i;
+	const char    *prm  = bevent_get_text(event);
+	struct ua     *ua   = bevent_get_ua(event);
 	int err = 0;
 	const char referto[] = "sip:user@127.0.0.1";
-	(void)call;
-	(void)prm;
 
 	ASSERT_TRUE(t != NULL);
 
 	if (ua != t->ua)
 		return;
 
-	if (ev == UA_EVENT_REGISTER_OK) {
+	if (ev == BEVENT_REGISTER_OK) {
 
 		info("event: Register OK!\n");
 
@@ -85,12 +84,12 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 		t->ua = mem_deref(t->ua);
 	}
-	else if (ev == UA_EVENT_REGISTER_FAIL) {
+	else if (ev == BEVENT_REGISTER_FAIL) {
 
 		err = EAUTH;
 		re_cancel();
 	}
-	else if (ev == UA_EVENT_REFER) {
+	else if (ev == BEVENT_REFER) {
 		ASSERT_STREQ(referto, prm);
 		++t->n_ev;
 	}
@@ -135,7 +134,7 @@ static int reg(enum sip_transp tp)
 	err = ua_register(t.ua);
 	TEST_ERR(err);
 
-	err = uag_event_register(ua_event_handler, &t);
+	err = bevent_register(event_handler, &t);
 	if (err)
 		goto out;
 
@@ -155,7 +154,7 @@ static int reg(enum sip_transp tp)
 	if (err) {
 		warning("selftest: ua_register test failed (%m)\n", err);
 	}
-	uag_event_unregister(ua_event_handler);
+	bevent_unregister(event_handler);
 	test_reset(&t);
 
 	return err;
@@ -166,6 +165,10 @@ int test_ua_register(void)
 {
 	int err = 0;
 
+#ifdef USE_TLS
+	conf_config()->sip.verify_server = false;
+#endif
+
 	err = ua_init("test", true, true, true);
 	TEST_ERR(err);
 
@@ -274,6 +277,8 @@ static int reg_dns(enum sip_transp tp)
 
 	memset(&t, 0, sizeof(t));
 
+	dnsc_cache_max(net_dnsc(net), 0);
+
 	/*
 	 * Setup server-side mocks:
 	 */
@@ -300,7 +305,7 @@ static int reg_dns(enum sip_transp tp)
 			goto out;
 		}
 
-		err = domain_add(t.srvv[0], domain);
+		err = domain_add(t.srvv[i], domain);
 		if (err)
 			goto out;
 
@@ -357,7 +362,7 @@ static int reg_dns(enum sip_transp tp)
 	err = ua_register(t.ua);
 	TEST_ERR(err);
 
-	err = uag_event_register(ua_event_handler, &t);
+	err = bevent_register(event_handler, &t);
 	if (err)
 		goto out;
 
@@ -380,7 +385,7 @@ static int reg_dns(enum sip_transp tp)
 	if (err) {
 		warning("selftest: ua_register test failed (%m)\n", err);
 	}
-	uag_event_unregister(ua_event_handler);
+	bevent_unregister(event_handler);
 
 	test_reset(&t);
 
@@ -463,7 +468,7 @@ static int reg_auth(enum sip_transp tp)
 	err = ua_register(t.ua);
 	TEST_ERR(err);
 
-	err = uag_event_register(ua_event_handler, &t);
+	err = bevent_register(event_handler, &t);
 	if (err)
 		goto out;
 
@@ -485,7 +490,7 @@ static int reg_auth(enum sip_transp tp)
 	if (err) {
 		warning("selftest: ua_register test failed (%m)\n", err);
 	}
-	uag_event_unregister(ua_event_handler);
+	bevent_unregister(event_handler);
 	test_reset(&t);
 
 	return err;
@@ -634,7 +639,7 @@ static int reg_auth_dns(enum sip_transp tp)
 	err = ua_register(t.ua);
 	TEST_ERR(err);
 
-	err = uag_event_register(ua_event_handler, &t);
+	err = bevent_register(event_handler, &t);
 	if (err)
 		goto out;
 
@@ -666,7 +671,7 @@ static int reg_auth_dns(enum sip_transp tp)
 	if (err) {
 		warning("selftest: ua_register test failed (%m)\n", err);
 	}
-	uag_event_unregister(ua_event_handler);
+	bevent_unregister(event_handler);
 
 	test_reset(&t);
 
@@ -897,7 +902,7 @@ static int test_ua_refer_base(enum sip_transp transp)
 
 	test_init(&t);
 
-	err = uag_event_register(ua_event_handler, &t);
+	err = bevent_register(event_handler, &t);
 	TEST_ERR(err);
 
 	err = ua_init("test",
@@ -944,7 +949,7 @@ static int test_ua_refer_base(enum sip_transp transp)
  out:
 	test_reset(&t);
 
-	uag_event_unregister(ua_event_handler);
+	bevent_unregister(event_handler);
 	ua_stop_all(true);
 	ua_close();
 
@@ -964,3 +969,47 @@ int test_ua_refer(void)
  out:
 	return err;
 }
+
+
+int test_ua_cuser(void)
+{
+	int err;
+	struct ua *ua1, *ua2, *ua3, *ua4;
+	uint32_t n_uas = list_count(uag_list());
+
+	/* make sure we dont have that UA already */
+	ASSERT_TRUE(NULL == uag_find_aor("sip:alice@test.invalid"));
+	ASSERT_TRUE(NULL == uag_find_aor("sip:bob@test.invalid"));
+
+	err = ua_alloc(&ua1, "Alice <sip:alice@test.invalid>;regint=0");
+	TEST_ERR(err);
+
+	err = ua_alloc(&ua2, "Alice <sip:alice@test.invalid>;regint=0");
+	TEST_ERR(err);
+
+	err = ua_alloc(&ua3, "Bob <sip:bob@test.invalid>;regint=0");
+	TEST_ERR(err);
+
+	err = ua_alloc(&ua4, "Bob <sip:bob@test.invalid>;regint=0");
+	TEST_ERR(err);
+
+	ASSERT_STREQ("alice", ua_cuser(ua1));
+	ASSERT_STREQ("bob", ua_cuser(ua3));
+
+	ASSERT_TRUE(0 != str_cmp(ua_cuser(ua1), ua_cuser(ua2)));
+	ASSERT_TRUE(0 != str_cmp(ua_cuser(ua3), ua_cuser(ua4)));
+
+	/* verify global UA keeper */
+	ASSERT_EQ((n_uas + 4), list_count(uag_list()));
+	ASSERT_TRUE(ua1 == uag_find_aor("sip:alice@test.invalid"));
+
+	mem_deref(ua1);
+	mem_deref(ua2);
+	mem_deref(ua3);
+	mem_deref(ua4);
+
+	ASSERT_EQ(n_uas, list_count(uag_list()));
+
+ out:
+	return err;
+}
